package com.yizhi.message.application.controller.util;

import com.yizhi.assignment.application.feign.AssignmentClient;
import com.yizhi.course.application.feign.CourseClient;
import com.yizhi.exam.application.feign.ExamAuthorizeClient;
import com.yizhi.exam.application.vo.exam.VisibleRangeExport;
import com.yizhi.live.application.feign.LiveActivityClient;
import com.yizhi.message.application.constans.Constans;
import com.yizhi.message.application.domain.MessageVisibleRange;

import com.yizhi.research.application.feign.ResearchAuthorizeClient;
import com.yizhi.system.application.system.remote.AccountClient;
import com.yizhi.system.application.system.remote.OrganizationClient;
import com.yizhi.system.application.vo.AccountVO;
import com.yizhi.system.application.vo.OrgVO;
import com.yizhi.training.application.feign.TrainingProjectClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Component
public class QueryAccount {
    @Autowired
    private AccountClient accountClient;
    @Autowired
    private OrganizationClient organizationClient;
    @Autowired
    private CourseClient courseClient;
    @Autowired
    private TrainingProjectClient trainingProjectClient;
    @Autowired
    private ExamAuthorizeClient examAuthorizeClient;
    @Autowired
    private ResearchAuthorizeClient researchAuthorizeClient;
    @Autowired
    private AssignmentClient assignmentClient;
    @Autowired
    private LiveActivityClient liveActivityClient;


    private QueryAccount() {
    }



    /**
     * 返回《accountId,account》的map
     *
     * @param accountIds
     * @return
     */
    public Map<Long, AccountVO> queryAccountByIds(List<Long> accountIds) {
        if (!CollectionUtils.isEmpty(accountIds)) {
            Map<Long, AccountVO> map = new HashMap<>(accountIds.size());
            List<AccountVO> accountVOS = accountClient.findByIds(accountIds);
            //组装map 以免双重循环
            if (!CollectionUtils.isEmpty(accountVOS)) {
                accountVOS.forEach(a -> {
                    if (!map.containsKey(a.getId())) {
                        map.put(a.getId(), a);
                    }
                });
            }
            return map;
        }

        return null;
    }

    /**
     * 返回《orgId,org》的map
     *
     * @param orgIds
     * @return
     */
    public Map<Long, OrgVO> queryOrgByIds(List<Long> orgIds) {
        if (!CollectionUtils.isEmpty(orgIds)) {
            Map<Long, OrgVO> map = new HashMap<>(orgIds.size());
            List<OrgVO> orgVOS = null;
            try {
                orgVOS = organizationClient.listByOrgIds(orgIds);
            } catch (Exception e) {
                e.printStackTrace();
                return null;
            }
            //组装map 以免双重循环
            if (!CollectionUtils.isEmpty(orgVOS)) {
                orgVOS.forEach(a -> {
                    if (!map.containsKey(a.getId())) {
                        map.put(a.getId(), a);
                    }
                });
            }
            return map;
        }
        return null;
    }


    /**
     * 全平台可见  根据站点id查询用户
     *
     * @param siteId
     * @return
     */
    public List<AccountVO> getAccountBySiteId(Long siteId) {
        List<AccountVO> accounts = new ArrayList<>();
        List<AccountVO> list = accountClient.findBySiteId(siteId);
        if (!CollectionUtils.isEmpty(list)) {
            accounts.addAll(list);
        }
        return accounts;
    }

    /**
     * 给定orgIds   accountIds   获取用户集合
     *
     * @param accountIds
     * @param orgIds
     * @param companyId
     * @return
     */
    public List<AccountVO> getAllAccounts(List<Long> accountIds, List<Long> orgIds, Long companyId) {
        List<AccountVO> accountVOS = new ArrayList<>();
        if (!CollectionUtils.isEmpty(orgIds)) {
            for (Long orgId : orgIds) {
                accountVOS.addAll(getAccountByOrgId(orgId, companyId));
            }
        }

        if (!CollectionUtils.isEmpty(accountIds)) {
            accountVOS.addAll(getAccountByIds(accountIds));
        }

        return accountVOS;
    }


    /**
     * 获取指定范围的人
     * 返回accountVo
     */
    public List<AccountVO> getMessageAccounts(List<MessageVisibleRange> list, Long companyId) {
        List<Long> orgIds = new ArrayList<>();
        List<Long> accountIds = new ArrayList<>();

        for (MessageVisibleRange range : list) {
            if (range.getType() == Constans.ORG) {
                orgIds.add(range.getRelationId());
            } else {
                accountIds.add(range.getRelationId());
            }
        }
        return getAllAccounts(accountIds, orgIds, companyId);
    }

    /**
     * 根据部门id 查询部门下所有人
     *
     * @param
     * @return
     */
    public List<AccountVO> getAccountByOrgId(Long orgId, Long companyId) {
        List<AccountVO> accounts = new ArrayList<>();
        List<AccountVO> list = accountClient.findByOrgId(orgId, companyId);
        if (!CollectionUtils.isEmpty(list)) {
            accounts.addAll(list);
        }
        return accounts;
    }

    /**
     * 根据用户id,查询用户对象
     *
     * @param accountIds
     * @return
     */
    public List<AccountVO> getAccountByIds(List<Long> accountIds) {
        List<AccountVO> accounts = new ArrayList<>();
        List<AccountVO> list = accountClient.findByIds(accountIds);
        if (!CollectionUtils.isEmpty(list)) {
            accounts.addAll(list);
        }
        return accounts;
    }


    /**
     * 根据业务获取相应用户信息
     *
     * @param taskType
     * @param taskId
     * @param companyId
     * @return
     */
    public List<AccountVO> getVisibleRangeByTask(Integer taskType, Long taskId, Long companyId) {


        switch (taskType) {
            // "关联模块类型(、1：学习计划、2：考试、3：调研、4、投票 5：报名、6：作业、7：签到、8：项目、9：直播、10：积分、11课程)"
            case 1:
                getVisibleRangeByTask(8, taskId, companyId);
                break;

            case 2:
                VisibleRangeExport examRange = examAuthorizeClient.exportVisibleRange(taskId);
                if (examRange != null) {
                    return getAllAccounts(examRange.getAccountIds(), examRange.getOrgIds(), companyId);
                }
            case 3:
                com.yizhi.research.application.vo.VisibleRangeExport researchRange = researchAuthorizeClient.exportVisibleRange(taskId);
                if (researchRange != null) {
                    return getAllAccounts(researchRange.getAccountIds(), researchRange.getOrgIds(), companyId);
                }
            case 4:
                break;
            case 5:
                getVisibleRangeByTask(8, taskId, companyId);
                break;

            case 6:
                com.yizhi.assignment.application.vo.VisibleRangeExport assignRange = assignmentClient.exportVisibleRange(taskId);
                if (assignRange != null) {
                    return getAllAccounts(assignRange.getAccountIds(), assignRange.getOrgIds(), companyId);
                }

            case 7:
                break;
            case 8:
                com.yizhi.training.application.vo.manage.VisibleRangeExport trainingRange = trainingProjectClient.exportVisibleRange(taskId);
                if (trainingRange != null) {
                    return getAllAccounts(trainingRange.getAccountIds(), trainingRange.getOrgIds(), companyId);
                }

            case 9:
                com.yizhi.live.application.vo.VisibleRangeExport liveRange = liveActivityClient.getListToMessage(taskId);
                if (liveRange != null) {
                    return getAllAccounts(liveRange.getAccountIds(), liveRange.getOrgIds(), companyId);
                }

            case 10:
                break;
            case 11:
                com.yizhi.course.application.vo.VisibleRangeExport courseRange = courseClient.vsibleRangeExport(taskId);
                if (courseRange != null) {
                    return getAllAccounts(courseRange.getAccountIds(), courseRange.getOrgIds(), companyId);
                }
        }
        return new ArrayList();
    }


}
