package com.yizhi.message.application.service.impl;

import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.service.impl.ServiceImpl;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.core.application.task.AbstractTaskHandler;
import com.yizhi.core.application.task.TaskExecutor;
import com.yizhi.message.application.constans.Constans;
import com.yizhi.message.application.domain.MessageAccount;
import com.yizhi.message.application.domain.MessageReadRecord;
import com.yizhi.message.application.mapper.MessageReadRecordMapper;
import com.yizhi.message.application.service.IMessageReadRecordService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author taoge123
 * @since 2020-03-29
 */
@Service
public class MessageReadRecordServiceImpl extends ServiceImpl<MessageReadRecordMapper, MessageReadRecord> implements IMessageReadRecordService {

    @Autowired
    private TaskExecutor taskExecutor;
    @Autowired
    private IdGenerator idGenerator;
    @Autowired
    private MessageReadRecordMapper messageReadRecordMapper;
    @Autowired
    private IMessageReadRecordService messageReadRecordService;

    @Override
    public Integer insertReadRecord(MessageAccount messageAccount) {
        RequestContext context = ContextHolder.get();
        taskExecutor.asynExecute(new AbstractTaskHandler() {
            @Override
            public void handle() {
                MessageReadRecord readRecord1 = new MessageReadRecord();
                readRecord1.setReaded(1);
                readRecord1.setAccountId(context.getAccountId());
                readRecord1.setMessageAccountId(messageAccount.getId());
                Integer readNum = messageReadRecordMapper.selectCount(new EntityWrapper<>(readRecord1));

                if (readNum == 0) {
                    MessageReadRecord readRecord = new MessageReadRecord();
                    readRecord.setId(idGenerator.generate());
                    readRecord.setMessageAccountId(messageAccount.getId());
                    readRecord.setVisibleRange(messageAccount.getVisibleRange());
                    readRecord.setAccountId(context.getAccountId());
                    readRecord.setCreateByName(context.getAccountName());
                    readRecord.setReaded(1);
                    readRecord.setReadTime(new Date());
                    messageReadRecordMapper.insert(readRecord);
                }
            }
        });
        return 1;
    }

    @Override
    public Map<Long, Long> getMap(List<Long> messageAccountIds) {
        Map<Long, Long> map = null;
        if (!CollectionUtils.isEmpty(messageAccountIds)) {
            map = new HashMap<>();
            RequestContext context = ContextHolder.get();
            MessageReadRecord readRecord = new MessageReadRecord();
            readRecord.setReaded(1);
            readRecord.setAccountId(context.getAccountId());
            EntityWrapper wrapper = new EntityWrapper(readRecord);
            wrapper.in("message_account_id", messageAccountIds);
            List<MessageReadRecord> readRecords = this.selectList(wrapper);
            if (!CollectionUtils.isEmpty(readRecords)) {
                for (MessageReadRecord a : readRecords) {
                    if (!map.containsKey(a.getMessageAccountId())) {
                        map.put(a.getMessageAccountId(), a.getAccountId());
                    }
                }
            }
        }
        return map;
    }

    @Override
    public Integer getReadNum() {
        RequestContext context = ContextHolder.get();
        MessageReadRecord readRecord = new MessageReadRecord();
        readRecord.setReaded(1);
        readRecord.setAccountId(context.getAccountId());
        readRecord.setVisibleRange(Constans.PUBLIC_SITE);
        EntityWrapper wrapper = new EntityWrapper(readRecord);
        return this.selectCount(wrapper);
    }

    @Override
    public Integer batchInsertReadRecord(List<MessageReadRecord> messageReadRecords) {
        if (CollectionUtils.isEmpty(messageReadRecords)) {
            return null;
        }
        taskExecutor.asynExecute(new AbstractTaskHandler() {
            @Override
            public void handle() {
                messageReadRecordService.insertBatch(messageReadRecords);
            }
        });
        return 1;
    }
}
